# printpop/__init__.py
# 
# """
# PrintPop PackageStyled console printing made simple — color, format, and clarity in one package.
# """

__version__ = "0.2.2"

from .core import print_indianred,print_lightcoral,print_salmon,print_darksalmon,print_lightsalmon,print_crimson,print_red,print_firebrick,print_darkred,print_pink,print_lightpink,print_hotpink,print_deeppink,print_mediumvioletred,print_palevioletred,print_lightsalmon,print_coral,print_tomato,print_orangered,print_darkorange,print_orange,print_gold,print_yellow,print_lightyellow,print_lemonchiffon,print_lightgoldenrodyellow,print_papayawhip,print_moccasin,print_peachpuff,print_palegoldenrod,print_khaki,print_darkkhaki,print_lavender,print_thistle,print_plum,print_violet,print_orchid,print_fuchsia,print_magenta,print_mediumorchid,print_mediumpurple,print_rebeccapurple,print_blueviolet,print_darkviolet,print_darkorchid,print_darkmagenta,print_purple,print_indigo,print_slateblue,print_darkslateblue,print_mediumslateblue,print_greenyellow,print_chartreuse,print_lawngreen,print_lime,print_limegreen,print_palegreen,print_lightgreen,print_mediumspringgreen,print_springgreen,print_mediumseagreen,print_seagreen,print_forestgreen,print_green,print_darkgreen,print_yellowgreen,print_olivedrab,print_olive,print_darkolivegreen,print_mediumaquamarine,print_darkseagreen,print_lightseagreen,print_darkcyan,print_teal,print_aqua,print_cyan,print_lightcyan,print_paleturquoise,print_aquamarine,print_turquoise,print_mediumturquoise,print_darkturquoise,print_cadetblue,print_steelblue,print_lightsteelblue,print_powderblue,print_lightblue,print_skyblue,print_lightskyblue,print_deepskyblue,print_dodgerblue,print_cornflowerblue,print_mediumslateblue,print_royalblue,print_blue,print_mediumblue,print_darkblue,print_navy,print_midnightblue,print_darkgray,print_darkgrey,print_dimgray,print_dimgrey,print_lightslategray,print_lightslategrey,print_lightgray,print_lightgrey,print_slategray,print_slategrey,print_gray,print_grey,print_darkslategray,print_darkslategrey,print_gainsboro,print_whitesmoke,print_silver,print_white,print_ghostwhite,print_snow,print_honeydew,print_mintcream,print_azure,print_aliceblue,print_floralwhite,print_seashell,print_oldlace,print_ivory,print_lavenderblush,print_linen,print_mistyrose,print_antiquewhite,print_bisque,print_blanchedalmond,print_wheat,print_cornsilk,print_brown,print_saddlebrown,print_sienna,print_chocolate,print_peru,print_sandybrown,print_burlywood,print_tan,print_rosybrown
from .core import print_indianred_back,print_lightcoral_back,print_salmon_back,print_darksalmon_back,print_lightsalmon_back,print_crimson_back,print_red_back,print_firebrick_back,print_darkred_back,print_pink_back,print_lightpink_back,print_hotpink_back,print_deeppink_back,print_mediumvioletred_back,print_palevioletred_back,print_lightsalmon_back,print_coral_back,print_tomato_back,print_orangered_back,print_darkorange_back,print_orange_back,print_gold_back,print_yellow_back,print_lightyellow_back,print_lemonchiffon_back,print_lightgoldenrodyellow_back,print_papayawhip_back,print_moccasin_back,print_peachpuff_back,print_palegoldenrod_back,print_khaki_back,print_darkkhaki_back,print_lavender_back,print_thistle_back,print_plum_back,print_violet_back,print_orchid_back,print_fuchsia_back,print_magenta_back,print_mediumorchid_back,print_mediumpurple_back,print_rebeccapurple_back,print_blueviolet_back,print_darkviolet_back,print_darkorchid_back,print_darkmagenta_back,print_purple_back,print_indigo_back,print_slateblue_back,print_darkslateblue_back,print_mediumslateblue_back,print_greenyellow_back,print_chartreuse_back,print_lawngreen_back,print_lime_back,print_limegreen_back,print_palegreen_back,print_lightgreen_back,print_mediumspringgreen_back,print_springgreen_back,print_mediumseagreen_back,print_seagreen_back,print_forestgreen_back,print_green_back,print_darkgreen_back,print_yellowgreen_back,print_olivedrab_back,print_olive_back,print_darkolivegreen_back,print_mediumaquamarine_back,print_darkseagreen_back,print_lightseagreen_back,print_darkcyan_back,print_teal_back,print_aqua_back,print_cyan_back,print_lightcyan_back,print_paleturquoise_back,print_aquamarine_back,print_turquoise_back,print_mediumturquoise_back,print_darkturquoise_back,print_cadetblue_back,print_steelblue_back,print_lightsteelblue_back,print_powderblue_back,print_lightblue_back,print_skyblue_back,print_lightskyblue_back,print_deepskyblue_back,print_dodgerblue_back,print_cornflowerblue_back,print_mediumslateblue_back,print_royalblue_back,print_blue_back,print_mediumblue_back,print_darkblue_back,print_navy_back,print_midnightblue_back,print_darkgray_back,print_darkgrey_back,print_dimgray_back,print_dimgrey_back,print_lightslategray_back,print_lightslategrey_back,print_lightgray_back,print_lightgrey_back,print_slategray_back,print_slategrey_back,print_gray_back,print_grey_back,print_darkslategray_back,print_darkslategrey_back,print_gainsboro_back,print_whitesmoke_back,print_silver_back,print_white_back,print_ghostwhite_back,print_snow_back,print_honeydew_back,print_mintcream_back,print_azure_back,print_aliceblue_back,print_floralwhite_back,print_seashell_back,print_oldlace_back,print_ivory_back,print_lavenderblush_back,print_linen_back,print_mistyrose_back,print_antiquewhite_back,print_bisque_back,print_blanchedalmond_back,print_wheat_back,print_cornsilk_back,print_brown_back,print_saddlebrown_back,print_sienna_back,print_chocolate_back,print_peru_back,print_sandybrown_back,print_burlywood_back,print_tan_back,print_rosybrown_back
from .core import print_formatted, print_bold, print_italic,print_underline,print_strikethrough,print_dim,print_blink,print_inverse,print_hidden,print_reset
from .core import print_hyperlink
from .core import print_rgb,print_color
from .core import main


